import Express from "express";
import lodash from "lodash";
import cors from "cors";
import bodyParser from "body-parser";
import assert from "assert";
import type { Request, Response, NextFunction } from "express";
import { Json } from "..";

type Config = {
  port: number;
};
// represents the actions service, a single function which maps to different callable functions
type Actions = (action: string, ...args: Json[]) => Promise<Json>;

export default async (config: Config, actions: Actions) => {
  assert(config.port, "requires express port");
  const app = Express();

  app.use(cors());
  app.use(bodyParser.json({ limit: "1mb" }));
  app.use(bodyParser.urlencoded({ extended: true }));

  app.get("/", (req: Request, res: Response) => {
    return res.send("ok");
  });

  app.post("/:action", (req: Request, res: Response, next: NextFunction) => {
    const action = req?.params?.action;
    actions(action, ...lodash.castArray(req.body))
      .then(res.json.bind(res))
      .catch(next);
  });

  app.use(cors());

  app.use(function (req: Request, res: Response, next: NextFunction) {
    next(new Error("Invalid Request"));
  });

  // this is an error handler, express knows this because the function has 4 parameters rather than 3
  // cant remove the "next" parameter, even though linting complains
  app.use(function (err: Error, req: Request, res: Response, next: NextFunction) {
    res.status(500).send(err.message || err);
  });

  return new Promise((res) => {
    app.listen(config.port, () => res(true));
  });
};
